import csv
from time import time

import requests
from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from trainvel.core.models import Station


class Command(BaseCommand):
    def handle(self, *args, **options):
        def convert_value(value: str) -> str:
            return True if value == 't' else False if value == 'f' else (value or None)

        stations = []

        STATIONS_URL = "https://raw.githubusercontent.com/trainline-eu/stations/master/stations.csv"
        with requests.get(STATIONS_URL, stream=True) as resp:
            for row in csv.DictReader(tqdm(resp.iter_lines(decode_unicode=True)), delimiter=';'):
                row: dict
                values = {k.replace(':', '_').replace('normalised_code', 'normalized_code_trainline')
                          .replace('same_as', 'same_as_id'): convert_value(v)
                          for k, v in row.items()}
                stations.append(Station(**values))

        Station.objects.all().delete()
        if options['verbosity'] >= 1:
            self.stdout.write("Deleted all stations.")
        start_time = time()
        with transaction.atomic():
            Station.objects.bulk_create(stations)
        if options['verbosity'] >= 1:
            self.stdout.write(f"Inserted {len(stations)} stations in {time() - start_time:.2f} seconds.")
